package com.six_group.sse.seni.guicontrols.checkboxcombo;

import java.util.Collections;

import javafx.application.Platform;
import javafx.beans.value.ObservableValue;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.scene.control.ComboBox;
import javafx.scene.control.ListCell;
import javafx.scene.control.ListView;
import javafx.util.Callback;

import com.sun.javafx.scene.control.ReadOnlyUnbackedObservableList;
import com.sun.javafx.scene.control.behavior.BehaviorBase;
import com.sun.javafx.scene.control.behavior.KeyBinding;
import com.sun.javafx.scene.control.skin.BehaviorSkinBase;
import com.sun.javafx.scene.control.skin.ComboBoxListViewSkin;

/**
 * Override from ControlsFX, just adds custom renderer functions
 * 
 * @author tkcr1
 *
 * @param <T>
 */
public class CheckBoxComboBoxSkin<T> extends BehaviorSkinBase<CheckBoxComboBox<T>, BehaviorBase<CheckBoxComboBox<T>>> {

	/**************************************************************************
	 * 
	 * Static fields
	 * 
	 **************************************************************************/

	/**************************************************************************
	 * 
	 * fields
	 * 
	 **************************************************************************/

	// visuals
	private final ComboBox<T>								comboBox;
	private final ListCell<T>								buttonCell;

	// data
	private final ObservableList<T>							items;
	private final ReadOnlyUnbackedObservableList<Integer>	selectedIndices;
	private final ReadOnlyUnbackedObservableList<T>			selectedItems;

	// tkcr1 - added
	private CheckBoxComboBox<T>								checkComboBox;

	/**************************************************************************
	 * 
	 * Constructors
	 * 
	 **************************************************************************/

	public CheckBoxComboBoxSkin(final CheckBoxComboBox<T> control) {
		super(control, new BehaviorBase<>(control, Collections.<KeyBinding> emptyList()));

		checkComboBox = control;

		this.items = control.getItems();

		selectedIndices = (ReadOnlyUnbackedObservableList<Integer>) control.getCheckModel().getSelectedIndices();
		selectedItems = (ReadOnlyUnbackedObservableList<T>) control.getCheckModel().getSelectedItems();

		comboBox = new ComboBox<T>(items) {
			protected javafx.scene.control.Skin<?> createDefaultSkin() {
				return new ComboBoxListViewSkin<T>(this) {
					// overridden to prevent the popup from disappearing
					@Override
					protected boolean isHideOnClickEnabled() {
						return false;
					}
				};
			}
		};

		// installs a custom CheckBoxListCell cell factory
		comboBox.setCellFactory(new Callback<ListView<T>, ListCell<T>>() {
			public ListCell<T> call(ListView<T> listView) {
				// tkcr1 - changed to our renderer
				return new CustomCheckBoxListCell<T>(checkComboBox, new Callback<T, ObservableValue<Boolean>>() {
					@Override
					public ObservableValue<Boolean> call(T item) {
						return control.getItemBooleanProperty(item);
					}
				});
			};
		});

		// we render the selection into a custom button cell, so that it can 
		// be pretty printed (e.g. 'Item 1, Item 2, Item 10').
		buttonCell = new ListCell<T>() {
			@Override
			protected void updateItem(T item, boolean empty) {
				// we ignore whatever item is selected, instead choosing
				// to display the selected item text using commas to separate
				// each item

				ICheckBoxComboBoxTitleRenderer<T> renderer = checkComboBox.getTitleRenderer();

				if (renderer == null) {
					final StringBuilder sb = new StringBuilder();
					for (int i = 0, max = selectedItems.size(); i < max; i++) {
						sb.append(selectedItems.get(i));
						if (i < max - 1) {
							sb.append(", ");
						}
					}
					setText(sb.toString());
				} else {
					// tkcr1 - added
					setText(renderer.getTitleString(selectedItems, selectedItems.size() == items.size()));
				}
			}
		};
		comboBox.setButtonCell(buttonCell);

		selectedIndices.addListener(new ListChangeListener<Integer>() {
			@Override
			public void onChanged(final Change<? extends Integer> c) {
				// we update the display of the ComboBox button cell by
				// just dumbly updating the index every time selection changes.
				buttonCell.updateIndex(1);
			}
		});

		getChildren().add(comboBox);

		// tkcr1 - added
		Platform.runLater(() -> buttonCell.updateIndex(1));
	}

}
