import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javafx.application.Platform;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Orientation;
import javafx.scene.Node;
import javafx.scene.control.ScrollBar;
import javafx.scene.control.SelectionMode;
import javafx.scene.layout.AnchorPane;

import org.controlsfx.control.spreadsheet.GridBase;
import org.controlsfx.control.spreadsheet.SpreadsheetCell;
import org.controlsfx.control.spreadsheet.SpreadsheetCellType;
import org.controlsfx.control.spreadsheet.SpreadsheetView;

public class DesktopView extends AnchorPane {

	private DesktopControler controler;
	private ArrayList<ObservableList<SpreadsheetCell>> theRows;
	private ObservableList<ObservableList<SpreadsheetCell>> theCells;
	private GridBase myGrid;
	private SpreadsheetView mySpread;
	private ScrollBar myScrollbar;
	private MyScrollbarListener myScrollListener;

	private IntegerProperty myValueRow;

	@FXML
	private AnchorPane main;

	public DesktopView(DesktopControler _controler) {

		this.controler = _controler;

		FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("desktop.fxml"));
		fxmlLoader.setRoot(this);
		fxmlLoader.setController(this);

		try {
			fxmlLoader.load();
		} catch (IOException exception) {
			throw new RuntimeException(exception);
		}

		this.myGrid = new GridBase(10, 5);
		this.theRows = new ArrayList<>();

		myValueRow = new SimpleIntegerProperty(0);
		myValueRow.addListener(new ChangeListener<Number>() {
			@Override
			public void changed(ObservableValue<? extends Number> arg0, Number _oldValue, Number _newValue) {
				System.out.println("CHANGEMENT DE VALUEROW / " + _oldValue + " -> " + _newValue);
				/*
				 * Ralimentation des donnes
				 */
				myScrollbar.valueProperty().removeListener(myScrollListener);
				updateGrille();
				myScrollbar.valueProperty().addListener(myScrollListener);
			}
		});

		List<String> enteteCols = new ArrayList<>();
		enteteCols.add("Colonne 1");
		enteteCols.add("Colonne 2");
		enteteCols.add("Colonne 3");
		enteteCols.add("Colonne 4");
		enteteCols.add("Colonne 5");
		myGrid.getColumnHeaders().addAll(enteteCols);

		/*
		 * On veut une grille de 10.000 lignes vides pour initialiser l'ascenseur
		 */
		System.out.println("J'initialise la grille  vide");
		for (int i = 0; i < 10000; i++) {
			ObservableList<SpreadsheetCell> aRow = FXCollections.observableArrayList();
			SpreadsheetCell cell1 = SpreadsheetCellType.STRING.createCell(i, 0, 1, 1, "");
			cell1.setGraphic(null);
			SpreadsheetCell cell2 = SpreadsheetCellType.STRING.createCell(i, 1, 1, 1, "");
			SpreadsheetCell cell3 = SpreadsheetCellType.STRING.createCell(i, 2, 1, 1, "");
			cell3.setGraphic(null);
			SpreadsheetCell cell4 = SpreadsheetCellType.STRING.createCell(i, 3, 1, 1, "");
			SpreadsheetCell cell5 = SpreadsheetCellType.STRING.createCell(i, 4, 1, 1, "");
			cell5.setGraphic(null);
			aRow.add(cell1);
			aRow.add(cell2);
			aRow.add(cell3);
			aRow.add(cell4);
			aRow.add(cell5);
			this.theRows.add(aRow);
		}

		/*
		 * Construction du Spreadsheet
		 */
		this.myGrid.setRows(this.theRows);
		this.mySpread = new SpreadsheetView(this.myGrid);
		this.mySpread.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
		for (int j = 0; j < 5; j++) {
			this.mySpread.getColumns().get(j).setPrefWidth(150d);
		}

		/*
		 * Alimentation des 3 premires lignes
		 */
		System.out.println("J'alimente les 10 premires lignes");
		for (int indexLig = 0; indexLig < 3; indexLig++) {

			System.out.println("ADD : " + indexLig);
			ObservableList<SpreadsheetCell> aRow = FXCollections.observableArrayList();
			SpreadsheetCell cell1 = SpreadsheetCellType.STRING.createCell(indexLig, 0, 1, 1, "" + indexLig);
			cell1.setGraphic(null);
			SpreadsheetCell cell2 = SpreadsheetCellType.STRING.createCell(indexLig, 1, 1, 1, "" + indexLig);
			SpreadsheetCell cell3 = SpreadsheetCellType.STRING.createCell(indexLig, 2, 1, 1, "" + indexLig);
			cell3.setGraphic(null);
			SpreadsheetCell cell4 = SpreadsheetCellType.STRING.createCell(indexLig, 3, 1, 1, "" + indexLig);
			SpreadsheetCell cell5 = SpreadsheetCellType.STRING.createCell(indexLig, 4, 1, 1, "" + indexLig);
			cell5.setGraphic(null);
			aRow.add(cell1);
			aRow.add(cell2);
			aRow.add(cell3);
			aRow.add(cell4);
			aRow.add(cell5);
			this.theRows.set(indexLig, aRow);
		}
		this.myGrid.setRows(this.theRows);
		this.mySpread.setGrid(this.myGrid);

		AnchorPane.setBottomAnchor(mySpread, 0d);
		AnchorPane.setTopAnchor(mySpread, 30d);
		AnchorPane.setRightAnchor(mySpread, 0d);
		AnchorPane.setLeftAnchor(mySpread, 0d);
		this.main.getChildren().add(mySpread);

		/*
		 * Dtection de la scrollbar
		 */
		Platform.runLater(new Runnable() {
			@Override
			public void run() {
				setScrollListener();
			}
		});
	}

	private void setScrollListener() {

		/*
		 * Dtection de la scrollbar
		 */
		/* Rcupration de la scrollbar verticale */
		for (Node n : this.mySpread.getSelectionModel().getTableView().lookupAll(".scroll-bar")) {
			if (n instanceof ScrollBar) {

				ScrollBar bar = (ScrollBar) n;
				if (bar.getOrientation().equals(Orientation.VERTICAL)) {

					myScrollbar = bar;
					System.out.println("J'ai trouv la scrollbar !!!");
				}
			}
		}
		/*
		 * Ajout du Listener de scroll
		 */
		this.myScrollListener = new MyScrollbarListener(this.myValueRow);
		this.myScrollbar.valueProperty().addListener(this.myScrollListener);

	}

	private void updateGrille() {

		int debut = this.myValueRow.get();
		int fin = debut + 3;
		for (int indexLig = 0; indexLig < fin; indexLig++) {

			System.out.println("ADD : " + indexLig);
			ObservableList<SpreadsheetCell> aRow = FXCollections.observableArrayList();
			SpreadsheetCell cell1 = SpreadsheetCellType.STRING.createCell(indexLig, 0, 1, 1, "" + indexLig);
			cell1.setGraphic(null);
			SpreadsheetCell cell2 = SpreadsheetCellType.STRING.createCell(indexLig, 1, 1, 1, "" + indexLig);
			SpreadsheetCell cell3 = SpreadsheetCellType.STRING.createCell(indexLig, 2, 1, 1, "" + indexLig);
			cell3.setGraphic(null);
			SpreadsheetCell cell4 = SpreadsheetCellType.STRING.createCell(indexLig, 3, 1, 1, "" + indexLig);
			SpreadsheetCell cell5 = SpreadsheetCellType.STRING.createCell(indexLig, 4, 1, 1, "" + indexLig);
			cell5.setGraphic(null);
			aRow.add(cell1);
			aRow.add(cell2);
			aRow.add(cell3);
			aRow.add(cell4);
			aRow.add(cell5);
			this.theRows.set(indexLig, aRow);
		}
		this.myGrid.setRows(this.theRows);
		this.mySpread.setGrid(this.myGrid);
	}
}
