package dk.ti.share5.client;

import static dk.ti.share5.client.gui.fonts.CommonFonts.CAPTION;
import static dk.ti.share5.client.gui.fonts.CommonFonts.HEADER;

import dk.ti.share5.client.gui.fileslist.FileListGUI;
import dk.ti.share5.client.tasks.BackgroundDataUpdateTask;
import javafx.animation.FadeTransition;
import javafx.application.Application;
import javafx.concurrent.Task;
import javafx.concurrent.Worker;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.geometry.Rectangle2D;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.PasswordField;
import javafx.scene.control.ProgressBar;
import javafx.scene.control.SplitPane;
import javafx.scene.control.TextField;
import javafx.scene.effect.DropShadow;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.text.Text;
import javafx.stage.Screen;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.util.Duration;

public class TestClient extends Application {

	public static final String APPLICATION_ICON = "http://cdn1.iconfinder.com/data/icons/Copenhagen/PNG/32/people.png";

	private static final int SPLASH_HEIGHT = 378;
	private static final int SPLASH_WIDTH = 576;

	public static void main(String[] args) {
		launch(args);
	}

	private FileListGUI listLeft;
	private FileListGUI listRight;
	private ProgressBar loadProgress;
	private PasswordField loginPwdTextField;
	private TextField loginUserTextField;
	private Stage mainStage;

	private Label progressText;

	private Stage splashStage;

	@Override
	public void init() {
	}

	@Override
	public void start(final Stage initStage) throws Exception {
		this.mainStage = initStage;

		this.mainStage.setTitle("TI Projekter - Login");
		this.mainStage.setScene(createLogin());
		this.mainStage.show();
	}

	private FileListGUI createFileList() {
		return new FileListGUI();
	}

	private Scene createLogin() {
		GridPane grid = new GridPane();
		grid.setAlignment(Pos.CENTER);
		grid.setHgap(10);
		grid.setVgap(10);
		grid.setPadding(new Insets(25, 25, 25, 25));

		Text scenetitle = new Text("Velkommen til TI Projekter");
		scenetitle.setFont(HEADER);
		grid.add(scenetitle, 0, 0, 2, 1);

		Label userName = new Label("Brugernavn:");
		userName.setFont(CAPTION);
		grid.add(userName, 0, 1);
		grid.add(loginUserTextField = new TextField(), 1, 1);
		loginUserTextField.setOnKeyPressed((e) -> {
			if(KeyCode.ENTER == e.getCode()) {
				doLogin();
			}
		});
		loginUserTextField.setPromptText("Angiv dit windows brugernavn.");
		loginUserTextField.setText("test");

		Label pw = new Label("Kodeord:");
		pw.setFont(CAPTION);
		grid.add(pw, 0, 2);
		grid.add(loginPwdTextField = new PasswordField(), 1, 2);
		loginPwdTextField.setOnKeyPressed((e) -> {
			if(KeyCode.ENTER == e.getCode()) {
				doLogin();
			}
		});
		loginPwdTextField.setPromptText("Angiv dit windows kodeord.");
		loginPwdTextField.setText("test");

		Button btn = new Button("Sign in");
		btn.setOnAction((e) -> doLogin());

		HBox hbBtn = new HBox(10);
		hbBtn.setAlignment(Pos.BOTTOM_RIGHT);
		hbBtn.getChildren().add(btn);
		grid.add(hbBtn, 1, 4);

		return new Scene(grid, 300, 275);
	}

	private void doLogin() {
		splash(mainStage);
	}

	private void showMainStage() {
		mainStage = new Stage(StageStyle.DECORATED);
		mainStage.setTitle("FXML Welcome");
		mainStage.getIcons().add(new Image(APPLICATION_ICON));
		mainStage.setScene(new Scene(splitpane()));

		splashStage.hide();
		splashStage = null;

		mainStage.show();
		mainStage.toFront();
	}

	private void showSplash(final Stage loginStage, Task<?> task, InitCompletionHandler initCompletionHandler) {
		splashStage = new Stage();

		ImageView splash = new ImageView(new Image(TestClient.class.getResourceAsStream("/splash.jpg")));
		loadProgress = new ProgressBar();
		loadProgress.setMaxWidth(SPLASH_WIDTH - 20);

		progressText = new Label("Initialiserer klient . . .");
		final VBox splashLayout = new VBox();
		splashLayout.getChildren().addAll(splash, loadProgress, progressText);

		progressText.setAlignment(Pos.CENTER);
		splashLayout.setEffect(new DropShadow());

		progressText.textProperty().bind(task.messageProperty());
		loadProgress.progressProperty().bind(task.progressProperty());
		task.stateProperty().addListener((observableValue, oldState, newState) -> {
			if(newState == Worker.State.SUCCEEDED) {
				loadProgress.progressProperty().unbind();
				loadProgress.setProgress(1);
				splashStage.toFront();
				FadeTransition fadeSplash = new FadeTransition(Duration.seconds(1.2), splashLayout);
				fadeSplash.setFromValue(1.0);
				fadeSplash.setToValue(0.0);
				fadeSplash.setOnFinished(actionEvent -> loginStage.hide());
				fadeSplash.play();

				initCompletionHandler.complete();
			} // todo add code to gracefully handle other task states.
		});
		task.exceptionProperty().addListener((observable, oldValue, newValue) -> {
			if(newValue != null) {
				Exception ex = (Exception)newValue;
				ex.printStackTrace();
			}
		});

		Scene splashScene = new Scene(splashLayout, Color.TRANSPARENT);
		final Rectangle2D screenBounds = Screen.getPrimary().getVisualBounds();
		splashStage.setScene(splashScene);
		splashStage.setX(screenBounds.getMinX() + screenBounds.getWidth() / 2 - SPLASH_WIDTH / 2);
		splashStage.setY(screenBounds.getMinY() + screenBounds.getHeight() / 2 - SPLASH_HEIGHT / 2);
		splashStage.initStyle(StageStyle.TRANSPARENT);
		// splashStage.setAlwaysOnTop(true);

		loginStage.hide();
		splashStage.show();
	}

	private void splash(final Stage initStage) {
		BackgroundDataUpdateTask backgroundDataTask = new BackgroundDataUpdateTask();

		showSplash(initStage, backgroundDataTask, () -> showMainStage());
		new Thread(backgroundDataTask).start();
	}

	private SplitPane splitpane() {
		SplitPane sp = new SplitPane(listLeft = createFileList(), listRight = createFileList());
		sp.setDividerPositions(0.5f, 0.5f);
		return sp;
	}

	public interface InitCompletionHandler {
		void complete();
	}

}