package dk.ti.share5.client.gui.fileinfo;

import java.util.ArrayList;
import java.util.List;

import org.controlsfx.validation.Severity;
import org.controlsfx.validation.ValidationResult;
import org.controlsfx.validation.ValidationSupport;
import org.controlsfx.validation.Validator;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Control;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Priority;

public class FileInfoGUI extends GridPane {

	private ComboBox<String> authorCombo;
	private TextField nameTxt;
	private ValidationSupport validationSupport;

	public FileInfoGUI() {
		initGridPane();
		initValidator();

		addName();
		addAuthor();
	}

	private void addAuthor() {
		ObservableList<String> users = getUsers();

		add(new Label("Forfatter:"), 0, 1);
		add(authorCombo = new ComboBox<>(), 1, 0);

		authorCombo.setPromptText("Vælg forfatter");
		authorCombo.setEditable(false);
		authorCombo.setItems(users);
		authorCombo.focusedProperty().addListener((arg0, oldPropertyValue, newPropertyValue) -> {
			if(newPropertyValue) {
				authorCombo.show();
			} else {
				authorCombo.hide();
			}
		});
	}

	private void addName() {
		add(new Label("Navn:"), 0, 0);
		add(nameTxt = new TextField(), 1, 0);
		setupTextfield(nameTxt, "Angiv navn");
	}

	private ObservableList<String> getUsers() {
		List<String> tmp = new ArrayList<>();
		tmp.add("User1");

		ObservableList<String> projects = FXCollections.observableArrayList();
		projects.addAll(tmp);
		return projects;
	}

	private void initGridPane() {
		ColumnConstraints column1 = new ColumnConstraints(100, 100, Double.MAX_VALUE);
		column1.setHgrow(Priority.ALWAYS);

		ColumnConstraints column2 = new ColumnConstraints(100);

		getColumnConstraints().addAll(column1, column2);
	}

	private void initValidator() {
		validationSupport = new ValidationSupport();
	}

	private void setupTextfield(TextField txt, String prompt) {
		txt.setPromptText(prompt);
		txt.focusedProperty().addListener((arg0, oldPropertyValue, newPropertyValue) -> {
			if(newPropertyValue) {
				txt.selectAll();
			}
		});

		Validator<String> validator = new Validator<String>() {

			@Override
			public ValidationResult apply(Control control, String value) {
				boolean empty = (value == null || value.trim().isEmpty());
				ValidationResult vr = ValidationResult.fromMessageIf(control, "skal angives", Severity.ERROR, empty);
				return vr;
			}
		};

		validationSupport.registerValidator(txt, validator);
	}
}