package dk.ti.share5.client.gui.fileslist;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import org.controlsfx.control.MasterDetailPane;

import dk.ti.share5.client.gui.fileinfo.FileInfoWizard;
import dk.ti.share5.client.gui.fileslist.structure.FileStructure;
import dk.ti.share5.client.gui.fileslist.structure.FileStructureContent;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.geometry.Side;
import javafx.scene.Node;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.CellDataFeatures;
import javafx.scene.control.TableView;
import javafx.scene.control.ToolBar;
import javafx.scene.input.Dragboard;
import javafx.scene.input.TransferMode;
import javafx.scene.layout.BorderPane;
import javafx.util.Callback;

public class FileListGUI extends BorderPane {

	private FileStructure fs;
	private ObservableList<String> projects;
	private ComboBox<String> selectProjectCombo;
	private TableView<FileStructureContent> table;

	public FileListGUI() {
		setTop(createToolbar());
		setCenter(createCenter());
	}

	private Node createCenter() {
		return new MasterDetailPane(Side.BOTTOM, createFileTable(), new Label("Detaljer"), true);
	}

	private TableView<FileStructureContent> createFileTable() {
		table = new TableView<>();
		table.getColumns().add(nameColumn());
		table.getColumns().add(extColumn());
		table.getColumns().add(sizeColumn());
		table.setDisable(false);
		table.setOnDragOver((event) -> {
			if(event.getGestureSource() != table && event.getDragboard().hasFiles()) {
				event.acceptTransferModes(TransferMode.COPY_OR_MOVE);
			}
			event.consume();
		});
		table.setOnDragDropped((event) -> {
			Dragboard db = event.getDragboard();
			boolean success = false;
			if(db.hasFiles()) {
				try {
					FileInfoWizard wizard = new FileInfoWizard(db.getFiles());
					// Platform.runLater(() -> wizard.show());
					wizard.show();
					success = true;
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			event.setDropCompleted(success);
			event.consume();
		});
		return table;
	}

	private ComboBox<String> createSelectCustomerCombo() {
		ObservableList<String> customers = getCustomers();

		ComboBox<String> selectCustomerCombo = new ComboBox<>();
		selectCustomerCombo.setPromptText("Vælg kunde...");
		selectCustomerCombo.setEditable(true);
		selectCustomerCombo.setItems(customers);
		selectCustomerCombo.focusedProperty().addListener((arg0, oldPropertyValue, newPropertyValue) -> {
			if(newPropertyValue) {
				selectCustomerCombo.show();
			} else {
				selectCustomerCombo.hide();
			}
		});
		selectCustomerCombo.setOnAction((event) -> {
			selectCustomer(null);
			selectProjectCombo.requestFocus();
		});
		return selectCustomerCombo;
	}

	private ComboBox<String> createSelectProjectCombo() {
		projects = FXCollections.observableArrayList();

		selectProjectCombo = new ComboBox<>();
		selectProjectCombo.setPromptText("Vælg projekt...");
		selectProjectCombo.setEditable(true);
		selectProjectCombo.setItems(projects);
		selectProjectCombo.focusedProperty().addListener((arg0, oldPropertyValue, newPropertyValue) -> {
			if(newPropertyValue) {
				selectProjectCombo.show();
			} else {
				selectProjectCombo.hide();
			}
		});
		selectProjectCombo.setOnAction((event) -> {
			selectProject(null);
		});
		return selectProjectCombo;
	}

	private ToolBar createToolbar() {
		ToolBar tb = new ToolBar();
		tb.getItems().add(createSelectCustomerCombo());
		tb.getItems().add(createSelectProjectCombo());
		return tb;
	}

	private TableColumn<FileStructureContent, String> extColumn() {
		return tableColumn("Ext", (cellData) -> cellData.getValue().getExtension());
	}

	private ObservableList<String> getCustomers() {
		List<String> tmp = new ArrayList<>();
		tmp.add("Customer 1");
		tmp.add("Customer 2");

		ObservableList<String> projects = FXCollections.observableArrayList();
		projects.addAll(tmp);
		return projects;
	}

	private TableColumn<FileStructureContent, String> nameColumn() {
		TableColumn<FileStructureContent, String> tableColumn = tableColumn("Navn", (cellData) -> cellData.getValue().getName());
		tableColumn.setPrefWidth(150);
		return tableColumn;
	}

	private void selectCustomer(String customer) {
		List<String> tmp = new ArrayList<>();
		tmp.add("Proj1");
		tmp.add("Proj2");
		this.projects.setAll(tmp);
	}

	private void selectProject(String project) {
		fs = new FileStructure();
		table.setItems(fs);
		table.setDisable(false);
	}

	private TableColumn<FileStructureContent, Number> sizeColumn() {
		TableColumn<FileStructureContent, Number> tableColumn = tableColumn("Størrelse", (cellData) -> cellData.getValue().getSize());
		tableColumn.setCellFactory(column -> {
			return new TableCell<FileStructureContent, Number>() {
				@Override
				protected void updateItem(Number item, boolean empty) {
					super.updateItem(item, empty);
					setText(formatSize((Long)item));
				}

				private String formatSize(Long size) {
					if(size <= 0) {
						return "0";
					}

					final String[] units = new String[] { "B", "kB", "MB", "GB", "TB" };
					int digitGroups = (int)(Math.log10(size) / Math.log10(1024));
					return new DecimalFormat("#,##0.#").format(size / Math.pow(1024, digitGroups)) + " " + units[digitGroups];
				}
			};
		});
		return tableColumn;
	}

	private <T> TableColumn<FileStructureContent, T> tableColumn(String caption, Callback<CellDataFeatures<FileStructureContent, T>, ObservableValue<T>> cellFactory) {
		TableColumn<FileStructureContent, T> tc = new TableColumn<>(caption);
		tc.setCellValueFactory(cellFactory);
		return tc;
	}

}
