package popover.test;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.geometry.Point2D;
import javafx.scene.Node;
import javafx.scene.control.PopupControl;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.layout.Background;
import javafx.scene.layout.BackgroundFill;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.util.Callback;

import org.controlsfx.control.PopOver;
import org.controlsfx.control.PopOver.ArrowLocation;

public class TestPopOverTableController implements Initializable {

	@FXML
	TableView<Widget>			widgetTableView;
	@FXML
	TableColumn<Widget, String>	valueOneColumn;
	@FXML
	TableColumn<Widget, String>	valueTwoColumn;
	@FXML
	TableColumn<Widget, String>	valueThreeColumn;

	ObservableList<Widget>		widgets	= FXCollections.observableArrayList();

	public TestPopOverTableController() {
		for (int i = 0; i < 10; i++) {
			widgets.add(new Widget());
		}
	}

	public void initialize(URL url, ResourceBundle resources) {
		widgetTableView.setOnMouseClicked(event -> {
			if (event.getClickCount() >= 2) {
				openSelectedItem((Node) event.getTarget());
			}
		});
		Callback<TableColumn<Widget, String>, TableCell<Widget, String>> redAndBlueFactory = new Callback<TableColumn<Widget, String>, TableCell<Widget, String>>() {

			@Override
			public TableCell<Widget, String> call(TableColumn<Widget, String> param) {
				TableCell<Widget, String> cell = new TableCell<Widget, String>() {

					@Override
					protected void updateItem(String item, boolean empty) {
						super.updateItem(item, empty);
						setText(item);
					}

				};
				cell.setStyle("-fx-text-fill : #ff0000; -fx-text-background-color : #0000ff");
				return cell;
			}
		};
		Callback<TableColumn<Widget, String>, TableCell<Widget, String>> greenAndWhiteFactory = new Callback<TableColumn<Widget, String>, TableCell<Widget, String>>() {

			@Override
			public TableCell<Widget, String> call(TableColumn<Widget, String> param) {
				TableCell<Widget, String> cell = new TableCell<Widget, String>() {

					@Override
					protected void updateItem(String item, boolean empty) {
						super.updateItem(item, empty);
						setText(item);
					}

				};
				cell.setStyle("-fx-text-fill : #008000");
				return cell;
			}
		};
		valueOneColumn.cellFactoryProperty().set(redAndBlueFactory);
		valueTwoColumn.cellFactoryProperty().set(greenAndWhiteFactory);
		// value three is intentionally left "normal" to illustrate difference

		valueOneColumn.cellValueFactoryProperty().set(new PropertyValueFactory<Widget, String>("valueOne"));
		valueTwoColumn.cellValueFactoryProperty().set(new PropertyValueFactory<Widget, String>("valueTwo"));
		valueThreeColumn.cellValueFactoryProperty().set(new PropertyValueFactory<Widget, String>("valueThree"));

		widgetTableView.setItems(widgets);
	}

	private void openSelectedItem(Node anchor) {
		Widget selectedWidget = widgetTableView.getSelectionModel().getSelectedItem();
		if (selectedWidget != null) {
			try {
				FXMLLoader loader = new FXMLLoader();
				loader.setLocation(TestPopoverApplication.class.getResource("TestPopOver.fxml"));
				loader.setController(new TestPopOverController(selectedWidget));
				GridPane testPopOverView = loader.load();
				// make it easier to see the data when showing as a PopupControl
				testPopOverView.setBackground(new Background(new BackgroundFill(Color.WHITE, null, null)));

				// TODO use one of the next two lines to show the data in a
				// PopOver or a PopupControl
				showAsPopOver(anchor, testPopOverView);
				// showAsPopupControl(anchor, testPopOverView);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	protected void showAsPopOver(Node anchor, GridPane pane) {
		PopOver popOver = new PopOver(pane);
		popOver.setAutoHide(true);
		popOver.setConsumeAutoHidingEvents(true);
		popOver.setArrowLocation(ArrowLocation.BOTTOM_CENTER);
		popOver.show(anchor);
	}

	protected void showAsPopupControl(Node anchor, GridPane pane) {
		PopupControl popupControl = new PopupControl();
		popupControl.getScene().setRoot(pane);
		popupControl.setAutoHide(true);
		popupControl.setConsumeAutoHidingEvents(true);
		Point2D point = anchor.localToScreen(anchor.getLayoutBounds().getMinX(), anchor.getLayoutBounds().getMaxY());
		popupControl.show(anchor, point.getX(), point.getY());
	}

}
