package de.fhg.iwes.javafx.vpp.frontend.module;

import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TextField;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

import org.controlsfx.validation.ValidationSupport;
import org.controlsfx.validation.Validator;

public class TestApplication extends Application {

	@Override
	public void start(Stage primaryStage) throws Exception {
		/**
		 * Data
		 */
		ObservableList<String> items = FXCollections.observableArrayList("No Item", "Item 1", "Item 2");

		/**
		 * Content
		 */
		VBox root = new VBox(10);
		root.setPadding(new Insets(10));

		ComboBox<String> comboBox = new ComboBox<>(items);
		comboBox.setPromptText("Please select...");
		root.getChildren().add(comboBox);

		TextField textField1 = new TextField();
		textField1.setPrefWidth(300);
		textField1.setPromptText("Only validated if \"Item 1\" is selected");
		root.getChildren().add(textField1);

		TextField textField2 = new TextField();
		textField2.setPrefWidth(300);
		textField2.setPromptText("Only validated if \"Item 2\" is selected");
		root.getChildren().add(textField2);

		/**
		 * Event
		 */
		comboBox.getSelectionModel().selectedItemProperty().addListener((o) -> {
			if (comboBox.getSelectionModel().getSelectedIndex() == 0) {
				textField1.setDisable(true);
				textField2.setDisable(true);
				textField1.setText("");
				textField2.setText("");
			} else if (comboBox.getSelectionModel().getSelectedIndex() == 1) {
				textField1.setDisable(false);
				textField2.setDisable(true);
				textField2.setText("");
			} else if (comboBox.getSelectionModel().getSelectedIndex() == 2) {
				textField1.setDisable(true);
				textField2.setDisable(false);
				textField1.setText("");
			}
		});

		/**
		 * Validation
		 */
		ValidationSupport validationSupport = new ValidationSupport();
		validationSupport.registerValidator(textField1, Validator.createEmptyValidator("textField1 not empty if \"Item 1\" selected."));
		validationSupport.registerValidator(textField2, Validator.createEmptyValidator("textField2 not empty if \"Item 2\" selected."));

		/**
		 * Start
		 */
		comboBox.getSelectionModel().select(0);
		primaryStage.setScene(new Scene(root));
		primaryStage.show();
	}

	public static void main(String[] args) {
		launch(args);
	}

}
