import javafx.application.Application;
import javafx.beans.property.SimpleStringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.geometry.Insets;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseButton;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.stage.Stage;
import javafx.util.Callback;

import org.controlsfx.control.PopOver;

/**
 * <p><b>Description: Example 13-6 w/ PopOver</b><br/>
 * </p>
 * Started with: http://docs.oracle.com/javase/8/javafx/user-interface-tutorial/table-view.htm#CJABHGAJ
 */
public class PopOverOnTableCell extends Application
{
  @Override
  public void start(Stage stage)
  {
    Scene scene = new Scene(new Group());
    stage.setTitle("Table View Sample");
    stage.setWidth(450);
    stage.setHeight(500);

    final Label label = new Label("Address Book");
    label.setFont(new Font("Arial", 20));

    TableColumn firstNameCol = new TableColumn("First Name");
    firstNameCol.setMinWidth(100);
    firstNameCol.setCellValueFactory(new PropertyValueFactory("firstName"));
    customize(firstNameCol);

    table.setItems(data);
    table.getColumns().addAll(firstNameCol);

    final VBox vbox = new VBox();
    vbox.setSpacing(5);
    vbox.setPadding(new Insets(10, 0, 0, 10));
    vbox.getChildren().addAll(label, table);

    ((Group) scene.getRoot()).getChildren().addAll(vbox);

    stage.setScene(scene);
    stage.show();
  }

  public void customize(TableColumn column)
  {
    column.setCellFactory(new Callback<TableColumn, TableCell>()
    {
      @Override
      public TableCell call(TableColumn param)
      {
        return new TableCell()
        {
          @Override
          protected void updateItem(Object item, boolean empty)
          {
            super.updateItem(item, empty);
            if (empty || item == null)
            {
              setText(null);
              setGraphic(null);
            }
            else
            {
              setText(item.toString());
            }

            setOnMouseClicked(event -> {
              if (event.getButton().equals(MouseButton.SECONDARY))
              {
                new PopOver().show(this);  // positions correctly but doesn't paint properly for some reason
                // the following alternate paints properly but isn't positioned as well:
                // new PopOver().show(column.getTableView(), event.getScreenX(), event.getScreenY());
              }
            });
          }
        };
      }
    });
  }
  private final TableView<Person> table = new TableView<>();
  private final ObservableList<Person> data = FXCollections.observableArrayList(new Person("Jacob"), new Person("Isabella"), new Person("Ethan"), new Person("Emma"), new Person("Michael"));

  public static class Person
  {
    private Person(String fName)
    {
      this.firstName = new SimpleStringProperty(fName);
    }
    public String getFirstName()
    {
      return firstName.get();
    }
    private final SimpleStringProperty firstName;
  }
}
