package sample;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Orientation;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import javafx.util.Callback;
import org.controlsfx.control.PopOver;

interface MyCallback {
  void call(String text);
}

class TextEntryPopOver extends PopOver {
  MyCallback callback = null;

  TextEntryPopOver(MyCallback cb) {
    callback = cb;
    final TextField field = new TextField();
    field.setPromptText("enter text");
    setContentNode(field);

    field.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        callback.call(field.getText());
        hide();
      }
    });
  }
}

class CustomPopover extends PopOver {
  CustomPopover() {
    final Button button = new Button("Click Here");
    final VBox vbox = new VBox(button);
    setContentNode(vbox);

    button.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        TextEntryPopOver tePopOver = new TextEntryPopOver(new MyCallback() {
          @Override
          public void call(String text) {
            vbox.getChildren().add(0, new TextArea());
          }
        });
        tePopOver.show(button);
      }
    });
  }
}

class CustomCell extends ListCell<String>{
  PopOver popover = new CustomPopover();

  CustomCell() {
    final CustomCell me = this;
    setOnMouseClicked(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
        popover.show(me);
      }
    });
  }

  @Override
  protected void updateItem(String item, boolean empty) {
    super.updateItem(item, empty);
    if(empty)
      setGraphic(null);
    else
      setGraphic(new Label(item));
  }
}

public class Main extends Application {
    @Override
    public void start(Stage primaryStage) throws Exception{
      primaryStage.setTitle("Popover Bug");
      ListView<String> list1 = new ListView<String>();
      list1.setCellFactory(new Callback<ListView<String>, ListCell<String>>() {
        @Override
        public ListCell<String> call(ListView<String> param) {
          return new CustomCell();
        }
      });
      ListView<String> list2 = new ListView<String>();
      list2.setCellFactory(new Callback<ListView<String>, ListCell<String>>() {
        @Override
        public ListCell<String> call(ListView<String> param) {
          return new CustomCell();
        }
      });

      list1.getItems().addAll("one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten");
      list2.getItems().addAll("one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten");
      SplitPane splitPane = new SplitPane(new VBox(list1, new Label("test")), list2);
      splitPane.setOrientation(Orientation.VERTICAL);
      primaryStage.setScene(new Scene(splitPane, 300, 275));
      primaryStage.show();
    }

    public static void main(String[] args) {
        launch(args);
    }
}
