package bugs.controlsfx.hiddensidespane;

import org.controlsfx.control.HiddenSidesPane;

import javafx.application.Application;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.layout.Background;
import javafx.scene.layout.BackgroundFill;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

/**
 * This application shows a bug in the HiddenSidesPane. According to the documentation the hidden pane
 * slides in when you move the mouse close enough to the corresponding side of the pane and 
 * "[...] will disappear again as soon as the mouse cursor exits it". This is not completely true
 * because it will also disappear when you click into the hidden pane. This behavior is especially
 * annoying if you place any controls like a button on the hidden pane because the pane will then disappear
 * after the first user interaction. The hidden pane should stay visible as long as the mouse cursor is
 * inside it independent of any mouse clicks or other user interactions with any controls inside the pane.
 */
public class HiddenSidesPaneBug extends Application {
	
	private BorderPane root;
	
	@Override
	public void start(final Stage stage) throws Exception {
        stage.setTitle(HiddenSidesPaneBug.class.getSimpleName());
        
        root = new BorderPane();
        Scene scene = new Scene(root, 800, 600);
        
        HiddenSidesPane hiddenSidesPane = new HiddenSidesPane();
        hiddenSidesPane.setContent(new BorderPane());
        
        HBox bottomPane = new HBox();
        bottomPane.setAlignment(Pos.CENTER);
        bottomPane.setPrefHeight(200);
        bottomPane.setBackground(new Background(new BackgroundFill(Color.BEIGE, null, null)));
        
        
        
        bottomPane.getChildren().add(new Button("Click me"));
        hiddenSidesPane.setBottom(bottomPane);
        
        root.setCenter(hiddenSidesPane);
        
        stage.setScene(scene);
        stage.show();
	}

	public static void main(String[] args) {
		launch(args);
	}
	
}
