package org.clemens.expandable;

import com.sun.javafx.css.converters.SizeConverter;
import javafx.beans.property.*;
import javafx.css.CssMetaData;
import javafx.css.SimpleStyleableIntegerProperty;
import javafx.css.Styleable;
import javafx.css.StyleableProperty;
import javafx.scene.Node;
import javafx.scene.control.Control;
import javafx.scene.control.Skin;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class ExpandablePane extends Control
{
	private static final String STYLE_CLASS = "expandable-pane";

	private static final class StyleableProperties
	{
		private static final CssMetaData<ExpandablePane, Number> ANIMATION_DURATION =
				new CssMetaData<ExpandablePane, Number>("-fx-animation-duration",
						SizeConverter.getInstance(), 250)
				{

					@Override
					public boolean isSettable(@Nullable final ExpandablePane expandable)
					{
						return !expandable.animationDuration.isBound();
					}

					@Override
					@SuppressWarnings("unchecked")
					public StyleableProperty<Number> getStyleableProperty(@Nonnull final ExpandablePane expandable)
					{
						return (StyleableProperty<Number>) expandable.animationDurationPropery();
					}
				};

		private static final List<CssMetaData<? extends Styleable, ?>> STYLEABLES;

		static
		{
			final List<CssMetaData<? extends Styleable, ?>> styleables = new ArrayList<>(Control.getClassCssMetaData());
			Collections.addAll(styleables,
					ANIMATION_DURATION
			);
			STYLEABLES = Collections.unmodifiableList(styleables);
		}
	}

	@Nonnull private final StringProperty title = new SimpleStringProperty();
	@Nonnull private final ObjectProperty<Node> graphic = new SimpleObjectProperty<>();

	@Nonnull private final ObjectProperty<Position> position = new SimpleObjectProperty<>(Position.TOP);
	@Nonnull private final BooleanProperty expanded = new SimpleBooleanProperty(true);

	@Nonnull private final ObjectProperty<Node> content = new SimpleObjectProperty<>();

	@Nonnull private final IntegerProperty animationDuration = new SimpleStyleableIntegerProperty(
			StyleableProperties.ANIMATION_DURATION,
			250);

	public ExpandablePane()
	{
		getStylesheets().add(ExpandablePane.class.getResource("expandable-pane.css").toExternalForm());
		getStyleClass().add(STYLE_CLASS);
	}

	@Override
	protected Skin<?> createDefaultSkin()
	{
		return new ExpandablePaneSkin(this);
	}

	public void setPosition(@Nonnull final Position position)
	{
		this.position.set(position);
	}

	@Nonnull
	public Position getPosition()
	{
		return position.get();
	}

	@Nonnull
	public ObjectProperty<Position> positionProperty()
	{
		return position;
	}

	public void setContent(@Nullable final Node node)
	{
		content.set(node);
	}

	@Nullable
	public Node getContent()
	{
		return content.get();
	}

	@Nonnull
	public ObjectProperty<Node> contentProperty()
	{
		return content;
	}

	public void setTitle(@Nonnull final String title)
	{
		this.title.setValue(title);
	}

	@Nonnull
	public String getTitle()
	{
		return title.get();
	}

	@Nonnull
	public StringProperty titleProperty()
	{
		return title;
	}

	@Nonnull
	public Node getGraphic()
	{
		return graphic.get();
	}

	@Nonnull
	public ObjectProperty<Node> graphicProperty()
	{
		return graphic;
	}

	public void setGraphic(@Nonnull final Node graphic)
	{
		this.graphic.set(graphic);
	}

	@SuppressWarnings("BooleanParameter")
	public void setExpanded(final boolean val)
	{
		expanded.set(val);
	}

	public boolean isExpanded()
	{
		return expanded.get();
	}

	@Nonnull
	public ReadOnlyBooleanProperty expandedProperty()
	{
		return expanded;
	}

	public void setAnimationDuration(final int duration)
	{
		animationDuration.set(duration);
	}

	public int getAnimationDuration()
	{
		return animationDuration.get();
	}

	@Nonnull
	public final IntegerProperty animationDurationPropery()
	{
		return animationDuration;
	}

	@Nonnull
	public static List<CssMetaData<? extends Styleable, ?>> getClassCssMetaData()
	{
		return StyleableProperties.STYLEABLES;
	}

	@Override
	@Nonnull
	public List<CssMetaData<? extends Styleable, ?>> getControlCssMetaData()
	{
		return getClassCssMetaData();
	}
}
