package org.clemens.expandable;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.BooleanPropertyBase;
import javafx.css.PseudoClass;
import javafx.event.ActionEvent;
import javafx.scene.control.Button;

public class ExpanderButton extends Button
{
	private static final PseudoClass COLLAPSED_PSEUDO_CLASS = PseudoClass.getPseudoClass("collapsed");
	private static final PseudoClass HORIZONTAL_EXPANSION_PSEUDO_CLASS = PseudoClass.getPseudoClass("horizontal");
	private static final PseudoClass INVERTED_OPENING_PSEUDO_CLASS = PseudoClass.getPseudoClass("inverted");

	private final BooleanProperty collapsed = new BooleanPropertyBase(false)
	{
		@Override
		protected void invalidated()
		{
			pseudoClassStateChanged(COLLAPSED_PSEUDO_CLASS, get());
		}

		@Override
		public Object getBean()
		{
			return ExpanderButton.this;
		}

		@Override
		public String getName()
		{
			return COLLAPSED_PSEUDO_CLASS.getPseudoClassName();
		}
	};

	private final BooleanProperty horizontal = new BooleanPropertyBase(false)
	{
		@Override
		protected void invalidated()
		{
			pseudoClassStateChanged(HORIZONTAL_EXPANSION_PSEUDO_CLASS, get());
		}

		@Override
		public Object getBean()
		{
			return ExpanderButton.this;
		}

		@Override
		public String getName()
		{
			return HORIZONTAL_EXPANSION_PSEUDO_CLASS.getPseudoClassName();
		}
	};

	private final BooleanProperty invertedOpening = new BooleanPropertyBase(false)
	{
		@Override
		protected void invalidated()
		{
			pseudoClassStateChanged(INVERTED_OPENING_PSEUDO_CLASS, get());
		}

		@Override
		public Object getBean()
		{
			return ExpanderButton.this;
		}

		@Override
		public String getName()
		{
			return INVERTED_OPENING_PSEUDO_CLASS.getPseudoClassName();
		}
	};

	@SuppressWarnings("ThisEscapedInObjectConstruction")
	public ExpanderButton()
	{
		addEventHandler(ActionEvent.ACTION, event -> invertExpansion());
		getStyleClass().add("expander-button");
	}

	@SuppressWarnings("BooleanParameter")
	public void setExpanded(final boolean expanded)
	{
		collapsed.set(!expanded);
	}

	private void invertExpansion()
	{
		collapsed.set(!collapsed.get());
	}

	public void setHorizontal()
	{
		horizontal.set(true);
	}

	public void setVertical()
	{
		horizontal.set(false);
	}

	public void useNormalOpening()
	{
		invertedOpening.set(false);
	}

	public void useInvertedOpening()
	{
		invertedOpening.set(true);
	}
}
