package org.clemens.expandable;

import com.sun.javafx.css.converters.SizeConverter;
import javafx.beans.property.*;
import javafx.css.*;
import javafx.scene.Node;
import javafx.scene.control.Control;
import javafx.scene.control.Skin;
import javafx.scene.input.MouseEvent;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class TitleBar extends Control
{
	private static final class StyleableProperties
	{
		private static final CssMetaData<TitleBar, Number> SPACING =
				new CssMetaData<TitleBar, Number>("-fx-spacing", SizeConverter.getInstance(), 0)
				{

					@Override
					public boolean isSettable(@Nullable final TitleBar control)
					{
						return !control.spacing.isBound();
					}

					@Override
					@SuppressWarnings("unchecked")
					public StyleableProperty<Number> getStyleableProperty(@Nonnull final TitleBar control)
					{
						return (StyleableProperty<Number>) control.spacingProperty();
					}
				};

		private static final List<CssMetaData<? extends Styleable, ?>> STYLEABLES;

		static
		{
			final List<CssMetaData<? extends Styleable, ?>> styleables = new ArrayList<>(Control.getClassCssMetaData());
			Collections.addAll(styleables,
					SPACING
			);
			STYLEABLES = Collections.unmodifiableList(styleables);
		}
	}

	private static final PseudoClass VERTICAL_PSEUDO_CLASS = PseudoClass.getPseudoClass("vertical");

	private final BooleanProperty vertical = new BooleanPropertyBase(false)
	{
		@Override
		protected void invalidated()
		{
			pseudoClassStateChanged(VERTICAL_PSEUDO_CLASS, get());
		}

		@Override
		public Object getBean()
		{
			return TitleBar.this;
		}

		@Override
		public String getName()
		{
			return VERTICAL_PSEUDO_CLASS.getPseudoClassName();
		}
	};

	@Nonnull private final DoubleProperty spacing = new SimpleStyleableDoubleProperty(StyleableProperties.SPACING, 0d);
	@Nonnull private final ObjectProperty<Position> position = new SimpleObjectProperty<>(Position.TOP);

	@Nonnull private final BooleanProperty expanded = new SimpleBooleanProperty(true);

	@Nonnull private final ObjectProperty<Node> graphic = new SimpleObjectProperty<>();
	@Nonnull private final StringProperty title = new SimpleStringProperty();

	public TitleBar()
	{
		addEventFilter(MouseEvent.MOUSE_ENTERED, e -> setHover(true));
		addEventFilter(MouseEvent.MOUSE_EXITED, e -> setHover(false));

		positionProperty().addListener((ov, o, n) -> {
			vertical.set(isVerticalPosition(n));
		});
	}

	@Override
	protected Skin<?> createDefaultSkin()
	{
		return new TitleBarSkin(this);
	}

	@Nonnull
	public Position getPosition()
	{
		return position.get();
	}

	@Nonnull
	public ObjectProperty<Position> positionProperty()
	{
		return position;
	}

	public void setPosition(@Nonnull final Position position)
	{
		this.position.set(position);
	}

	@Nonnull
	public Node getGraphic()
	{
		return graphic.get();
	}

	@Nonnull
	public ObjectProperty<Node> graphicProperty()
	{
		return graphic;
	}

	public void setGraphic(@Nonnull final Node graphic)
	{
		this.graphic.set(graphic);
	}

	@Nonnull
	public String getTitle()
	{
		return title.get();
	}

	@Nonnull
	public StringProperty titleProperty()
	{
		return title;
	}

	public void setTitle(@Nonnull final String title)
	{
		this.title.set(title);
	}

	@Nonnull
	public double getSpacing()
	{
		return spacing.get();
	}

	@Nonnull
	public DoubleProperty spacingProperty()
	{
		return spacing;
	}

	public void setSpacing(@Nonnull final double spacing)
	{
		this.spacing.set(spacing);
	}

	@Nonnull
	public boolean getExpanded()
	{
		return expanded.get();
	}

	@Nonnull
	public BooleanProperty expandedProperty()
	{
		return expanded;
	}

	@SuppressWarnings("BooleanParameter")
	public void setExpanded(final boolean expanded)
	{
		this.expanded.set(expanded);
	}

	void invertExpansion()
	{
		expanded.set(!expanded.get());
	}

	@Nonnull
	public static List<CssMetaData<? extends Styleable, ?>> getClassCssMetaData()
	{
		return StyleableProperties.STYLEABLES;
	}

	@Override
	@Nonnull
	public List<CssMetaData<? extends Styleable, ?>> getControlCssMetaData()
	{
		return getClassCssMetaData();
	}

	static boolean isHorizontalPosition(@Nonnull final Position pos)
	{
		return pos == Position.BOTTOM || pos == Position.TOP;
	}

	static boolean isVerticalPosition(@Nonnull final Position pos)
	{
		return !isHorizontalPosition(pos);
	}
}
